#include "../../include/database.h"
#include "../../include/error.h"
#include "../../include/util.h"
#include "../../include/ctx.h"

#include <string.h>

lm_database_t *lm_ctx_database_new(lm_ctx_t *ctx){
  char dbpath[strlen(ctx->data)+10];
  join(dbpath, ctx->data, "db");
  return lm_database_new(dbpath);
}

bool lm_ctx_database_is_installed(lm_ctx_t *ctx, lm_pkg_t *pkg, bool check_version){
  lm_database_t *db = lm_ctx_database_new(ctx);
  lm_pkg_t found;
  bool ret = false;

  if(NULL == db)
    return false;

  if(!lm_database_find(db, &found, pkg->name))
    goto end;

  if(check_version && !eq(found.version, pkg->version))
    goto end;

  ret = true;
end:
  lm_database_free(db);
  return ret;
}

bool lm_ctx_database_find(lm_ctx_t *ctx, lm_pkg_t *pkg, char *name){
  if(NULL == ctx || NULL == pkg || NULL == name){
    lm_error_set(LM_ERR_ArgNULL);
    return false;
  }

  lm_database_t *db = lm_ctx_database_new(ctx);
  if(!lm_database_find(db, pkg, name))
    return false;
  return true;
}

bool lm_ctx_database_foreach(lm_ctx_t *ctx, lm_ctx_database_callback_t callback, void *data){
  if(NULL == ctx || NULL == callback){
    lm_error_set(LM_ERR_ArgNULL);
    return false;
  }

  lm_database_t *db = lm_ctx_database_new(ctx);
  lm_pkg_t pkg;

  while(lm_database_next(db, &pkg)){
    if(!callback(ctx, &pkg, data))
      break;
  }

  return true;
}
